@@ -1,5 +1,5 @@
 /* ARC-specific support for 32-bit ELF
-   Copyright (C) 1994-2025 Free Software Foundation, Inc.
+   Copyright (C) 1994-2026 Free Software Foundation, Inc.
    Contributed by Cupertino Miranda (cmiranda@synopsys.com).
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -556,7 +556,7 @@ arc_extract_features (const char *p)
 
   for (i = 0; i < ARRAY_SIZE (bfd_feature_list); i++)
     {
-      char *t = strstr (p, bfd_feature_list[i].attr);
+      const char *t = strstr (p, bfd_feature_list[i].attr);
       unsigned l = strlen (bfd_feature_list[i].attr);
       if ((t != NULL)
 	  && (t[l] == ','
